# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['detextify']

package_data = \
{'': ['*'], 'detextify': ['config/*']}

install_requires = \
['absl-py>=1.3.0,<2.0.0',
 'accelerate>=0.15.0,<0.16.0',
 'azure-cognitiveservices-vision-computervision>=0.9.0,<0.10.0',
 'diffusers>=0.10.2,<0.11.0',
 'numpy>=1.23.5,<2.0.0',
 'openai>=0.25.0,<0.26.0',
 'opencv-python>=4.6.0.66,<5.0.0.0',
 'paddleocr>=2.6.1.1,<3.0.0.0',
 'paddlepaddle>=2.4.1,<3.0.0',
 'pillow>=9.3.0,<10.0.0',
 'replicate>=0.4.0,<0.5.0',
 'requests>=2.28.1,<3.0.0',
 'torch>=1.13.0,<2.0.0',
 'transformers>=4.25.1,<5.0.0']

setup_kwargs = {
    'name': 'detextify',
    'version': '0.1.0',
    'description': '',
    'long_description': 'Library to remove unwanted pseudo-text from images generated by AI\n===\nTo run the main script (which will eventually be removed from the library):\npoetry run python detextify/main.py\n',
    'author': 'Julia',
    'author_email': 'turc.raluca@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
